// WtPickers.js
//
// Copyright  2005, glenn andreas.	 All rights reserved
//

protoSelectPicker = View({
	_proto : protoView,
	viewClass : "selectPicker",
	CreateElement : function() {
		var retval = document.createElement('select');
		for (var i=0;i<this.pickItems.length;i+=1) {
			var option = document.createElement('option');
			retval.appendChild(option);
			var item = this.pickItems[i];
			if (typeof item == typeof "") {
				option.innerText = item;
				option.value = i;
			} else if (typeof item == typeof {}) {
				option.innerText = item.item;
				if (item.value || item.value === null) {
					option.value = item.value;
				} else {
					option.value = i;
				}
			}
		}
		return retval;
	},
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		var self = this;
		this.viewElement.onchange = function() {
			self.PickActionScript(self.viewElement.value);
		}
	},
	ViewChangedScript : function(symbol,value) {
		if (symbol == 'value') {
			// format the value
			this.viewElement.value = value;
		}
	},
	PickActionScript : nullfunc
});

protoInputCheckbox = View({
	_proto : protoView,
	viewClass : "inputCheckbox",
	CreateElement : function() {
		// a check box doesn't actually have text, so we make a div with both
		var retval = document.createElement('div');
		this.checkboxElement = document.createElement('input');
		this.checkboxElement.setAttribute('type','checkbox');
		retval.appendChild(this.checkboxElement);
		this.titleElement = document.createTextNode(this.text);
		retval.appendChild(this.titleElement);
		return retval;
	},
	ValueChanged : nullfunc,
	buttonValue : true, // the value when selected
	viewValue : false,
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		if (this.text) {
			this.titleElement.innerText = this.text;
		}
		var self = this;
		this.checkboxElement.onchange = function() {
			if (self.checkboxElement.checked) {
				self.viewValue = self.buttonValue;
			} else {
				self.viewValue = null;
			}
			self.ValueChanged(self.viewValue);
		};
	},
	ViewChangedScript : function(symbol,value) {
		if (symbol == 'viewValue') {
			// format the value
			this.checkboxElement.checked = (value == this.buttonValue);
		}
	}
});

protoInputSlider = View({
	_proto : protoView,
	viewClass : "inputSlider",
	CreateElement: function() {
		var retval = document.createElement('input');
		retval.setAttribute('type','range');
		var self = this;
		retval.onclick = function() { // onclick doesn't provide live feedback, unforuntately
			//alert("on change slider");
			self.value = self.viewElement.value;
			self.ValueChanged(self.value);
		};
		return retval;
	},
	ValueChanged : nullfunc,
	min : 0.0,
	max : 1.0,
	value : 0.5,
	step : 0.1,
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		this.viewElement.setAttribute('min', this.min);
		this.viewElement.setAttribute('max', this.max);
		this.viewElement.setAttribute('value', this.value);
		this.viewElement.setAttribute('step', this.step);
	},
	ViewChangedScript : function(symbol,value) {
		if (symbol == 'value') {
			this.viewElement.setAttribute('value', this.value);
		}
	}
});

protoTextInput = View({
	_proto : protoView,
	viewClass : "textInput",
	viewChangedScript : nullfunc,
	textSetup : nullfunc,
	textChanged : nullfunc,
	CreateElement : function() {
		var retval = document.createElement('input');
		var text = this.textSetup();
		if (!text && text !== '') {
			text = this.text;
		}
		if (text) {
			retval.value = this.text;
		}
		var self = this;
		retval.onchange = function() {
			self.text = self.viewElement.value;
			//self.ViewChangedScript();
			self.textChanged();
		};
		return retval;
	},
	ViewChangedScript : function( slotSymbol, value) {
		if (slotSymbol == 'text') {
			this.viewElement.value = this.text;
		}
	}
});
protoTextArea = View({
	_proto : protoView,
	viewClass : "textArea",
	viewChangedScript : nullfunc,
	textSetup : nullfunc,
	textChanged : nullfunc,
	CreateElement : function() {
		var retval = document.createElement('textarea');
		var text = this.textSetup();
		if (!text && text !== '') {
			text = this.text;
		}
		if (text) {
			retval.value = this.text;
		}
		var self = this;
		retval.onchange = function() {
			self.text = self.viewElement.value;
			//self.ViewChangedScript();
			self.textChanged();
		};
		return retval;
	},
	ViewChangedScript : function( slotSymbol, value) {
		if (slotSymbol == 'text') {
			this.viewElement.value = this.text;
		}
	}
});

protoControl = View({
	_proto : protoView,
	viewClass : "control",
	viewFlags : protoView.viewFlags | vClickable,
	PrepareForClick : function() {},
	ClickDone : function() {},
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		// make sure dashboard region is set so we can track correctly
		this.viewElement.style.appleDashboardRegion = 'dashboard-region(control rectangle)';
	},
	ViewClickScript : function(event) {
		this.TrackHilite(event);
	}
});


protoPictureButton = View({
	_proto : protoControl,
	viewClass : "pictureButton",
	elementClass : "img",
	ViewSetupChildrenScript : function() {
		protoControl.ViewSetupChildrenScript.call(this);
		if (!this.viewElement.src) {
			if (this.imageSrc) {
				this.viewElement.src = this.imageSrc;
			} else if (this.icon) {
				if (typeof this.icon == "string") {
					this.viewElement.src = this.icon + "_plain.png";
				} else {
					this.viewElement = this.icon; // assume to be an img
				}
			}
		}
	},
	ViewHiliteScript : function(onOff) {
		protoControl.ViewHiliteScript.call(this,onOff);
		if (onOff) {
			if (this.imageSrcHilite) {
				this.viewElement.src = this.imageSrcHilite;
			} else if (this.icon) {
				if (typeof this.icon == "string") {
					this.viewElement.src = this.icon + "_hilite.png";
				} else {
					this.viewElement = this.icon; // assume to be an img
				}
			}
		} else {
			if (this.imageSrc) {
				this.viewElement.src = this.imageSrc;
			} else if (this.icon) {
				if (typeof this.icon == "string") {
					this.viewElement.src = this.icon + "_plain.png";
				} else {
					this.viewElement = this.icon; // assume to be an img
				}
			}
		}
	},
	ViewClickScript : function(event) {
		this.TrackButton(event);
	}
});

protoPictureRadioButton = View({
	_proto : protoPictureButton,
	viewClass : "pictureRadioButton",
	buttonValue : true,
	viewValue : null,
	ViewHiliteScript : function(onOff) {
		protoPictureButton.ViewHiliteScript.call(this,onOff);
		if (onOff) {
			this.viewValue = this.buttonValue;
		} else {
			this.viewValue = null;
		}
	},
	ViewShowScript : function() {
		protoPictureButton.ViewShowScript.call(this);
		if (this.viewValue !== null) {
			this.HiliteUnique(true);
		}
	},
	ButtonClickScript : function() {
		this.HiliteUnique(true);
		this._parent.SetClusterValue(this.buttonValue);
	},
	ViewClickScript : function(event) {
		if (this.viewValue === null) {
			this.TrackButton(event);
		} // if already on, stays on
	}
});

protoRadioCluster = View({
	_proto : protoView,
	viewClass : "radioCluster",
	clusterValue : null,
	InitClusterValue : function(buttonValue) {
		var children = this.ChildViewFrames();
		for (var i=0;i<children.length;i+=1) {
			var child = children[i];
			if ((child.buttonValue || child.buttonValue === false || child.buttonValue === 0) && child.buttonValue == buttonValue) {
				child.Hilite(true);
			} else {
				child.Hilite(false);
			}
		}
		this.clusterValue = buttonValue;
	},
	ViewSetupDoneScript : function() {
		protoView.ViewSetupDoneScript.call(this);
		this.InitClusterValue(this.clusterValue);
	},
	ClusterChanged : nullfunc,
	SetClusterValue : function(buttonValue) {
		this.InitClusterValue(buttonValue);
		this.ClusterChanged();
	}
});


// protoNumberTyper is a small replacement for protoNumberPicker
protoNumberTyper = View({
	_proto : protoControl,
	viewClass : 'numberTyper',
	viewFlags : protoControl.viewFlags | vKeyable,
	chromeStyle : 'monoFont mediumFont',
	minValue : 0,
	maxValue : 9,
	wrapAround : false,
	value : null,
	// emulate protoNumberPicker
	ViewSetupChildrenScript : function() {
		protoControl.ViewSetupChildrenScript.call(this);
		if (this.value !== null) {
			SetValue(this, 'value', this.value);
		} else {
			SetValue(this, 'value', this.minValue);
		}
	},
	ViewChangedScript : function(symbol,value) {
		if (symbol == 'value') {
			// format the value
			if (this.value < this.minValue) { this.value = this.minValue; }
			if (this.value > this.maxValue) { this.value = this.maxValue; }
			if (this.showLeadingZeros) {
				var maxLength = (this.maxValue + "").length;
				var strValue = this.value + "";
				while (strValue.length < maxLength) {
					strValue = "0" + strValue;
				}
				this.viewElement.innerText = strValue;
			} else {
				this.viewElement.innerText = this.value + "";
			}
		} else if (symbol == 'maxValue') {
			if (this.value > this.maxValue) {
				SetValue(this, 'value', this.maxValue);
				this.ClickDone();
			}
		} else if (symbol == 'minValue') {
			if (this.value < this.minValue) {
				SetValue(this, 'value', this.minValue);
				this.ClickDone();
			}
		}
	},
	CalcDeltaValue : function(delta) {
		var newValue = this.value + delta;
		if (newValue < this.minValue) {
			if (this.wrapAround) {
				newValue = this.maxValue;
			} else {
				newValue = this.minValue;
			}
		} else if (newValue > this.maxValue) {
			if (this.wrapAround) {
				newValue = this.minValue;
			} else {
				newValue = this.maxValue;
			}
		}
		return newValue;
	},
	KeyEvent : function(event) {
		event.stopPropagation();
		if (48 <= event.charCode && event.charCode <= 57) {
			this.PrepareForClick();
			var digit = event.charCode - 48;
			var value = this.value * 10 + digit;
			if (value > this.maxValue) {
				value = digit; // reset to single digit?
			}
			if (value > this.maxValue) {
				value = this.minValue;
			}
			SetValue(this, 'value', value);
			this.ClickDone();
			return true;
		} else if (event.keyIdentifier == "Up") { // key up
			var newValue = this.CalcDeltaValue(1);
			if (newValue != this.value) {
				this.PrepareForClick();
				SetValue(this,'value',newValue);
				this.ClickDone();
				return true;
			}
		} else if (event.keyIdentifier == "Down") { // down
			var newValue = this.CalcDeltaValue(-1);
			if (newValue != this.value) {
				this.PrepareForClick();
				SetValue(this,'value',newValue);
				this.ClickDone();
				return true;
			}
		} else if (event.charCode == 8) { // backspace
			this.PrepareForClick();
			SetValue(this, 'value', this.minValue);
			this.ClickDone();
			return true;
		} else {
			alert(event.charCode);
		}
		return false;
	}
});

protoListTyper = View({
	_proto : protoControl,
	viewClass : 'listTyper',
	viewFlags : protoControl.viewFlags | vKeyable,
	chromeStyle : 'userFont mediumFont',
	value : null,
	caseSensitive : false,
	pickItems : ['dummy'],
	pickItemIndex : 0,
	ViewSetupChildrenScript : function() {
		protoControl.ViewSetupChildrenScript.call(this);
		if (this.value !== null) {
			SetValue(this, 'value', this.value);
		} else {
			SetValue(this, 'value', this.pickItems[this.pickItemIndex]);
		}
	},
	ViewChangedScript : function(symbol,value) {
		if (symbol == 'value') {
			// format the value
			this.value = value;
			this.viewElement.innerText = this.value;
			this.pickItemIndex = this.pickItems.findFirst(value);
		}
	},
	PreviousValue  : function() {
		if (this.pickItemIndex < this.pickItems.length - 1) {
			this.pickItemIndex += 1;
		} else {
			this.pickItemIndex = 0;
		}
		SetValue(this, 'value', this.pickItems[this.pickItemIndex]);
	},
	NextValue : function() {
		if (this.pickItemIndex > 0) {
			this.pickItemIndex -= 1;
		} else {
			this.pickItemIndex = this.pickItems.length-1;
		}
		SetValue(this, 'value', this.pickItems[this.pickItemIndex]);
	},
	KeyEvent : function(event) {
		if (event.keyIdentifier == "Up") { // key up
			this.PrepareForClick();
			this.PreviousValue();
			this.ClickDone();
			return true;
		} else if (event.keyIdentifier == "Down") { // down
			this.PrepareForClick();
			this.NextValue();
			this.ClickDone();
			return true;
		} else {
			var s0 = String.fromCharCode(event.charCode);
			if (!this.caseSensitive) {
				s0 = s0.toLowerCase();
			}
			var found = null;
			for (var i=0;i<this.pickItems.length;i++) {
				if (this.caseSensitive) {
					if (this.pickItems[i].slice(0,1) == s0) {
						found = i;
						break;
					}
				} else {
					if (this.pickItems[i].slice(0,1).toLowerCase() == s0) {
						found = i;
						break;
					}
				}
			}
			if (found !== null) {
				this.PrepareForClick();
				this.pickItemIndex = found;
				SetValue(this, 'value', this.pickItems[this.pickItemIndex]);
				this.ClickDone();
				return true;
			}
		}
		return false;
	}
});


protoTimeTyper = View({
	_proto : protoView,
	viewClass : 'timeTyper',
	date : null,
	RefreshDate : function() {
		if (this.date.getHours() < 12) {
			SetValue(this.hours, 'value', this.date.getHours());
			SetValue(this.ampm, 'value','AM');
		} else {
			SetValue(this.hours, 'value', this.date.getHours() - 12);
			SetValue(this.ampm, 'value','PM');
		}
		SetValue(this.minutes, 'value', this.date.getMinutes());
	},
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		if (!this.date) {
			this.date = new Date();
		}
		//this.RefreshDate();
		this.LayoutChildrenMatrix({
			tabAcross: 5,
			tabDown: 1,
			tabWidths: this.chromeChildWidths,
			tabHeights: this.chromeHeight - 2,
			tabChildren: [this.hours, this.colon, this.minutes,null, this.ampm]
		},1,1);
	},
	chromeHeight : 18,
	chromeChildWidths : [18,/* Hours */
	8,	/* : */
	20,	 /* minutes */
	5,	/* */
	25	/* AM/PM */
	],
	chromeWidth : 18 + 8 + 20 + 5 + 25 + 2,
	SetHours : function(value) {
		if (this.ampm.value == 'AM') {
			this.date.setHours(value);
		} else {
			this.date.setHours(value + 12);
		}
		this.NewTime(this.date);
	},
	SetMinutes : function(value) {
		this.date.setMinutes(value);
		this.NewTime(this.date);
	},
	SetAmPm : function(value) {
		if (value == 'AM') {
			this.date.setHours(this.hours.value);
		} else {
			this.date.setHours(this.hours.value + 12);
		}
		this.NewTime(this.date);
	},
	ViewChangedScript : function(symbol,value) {
		if (symbol == 'date') {
			// format the value
			this.date = value;
			this.RefreshDate();
		}
	},
	NewTime : function(date) {
	}
});
DeclareChild(View({
	_proto: protoNumberTyper,
	minValue : 1,
	maxValue : 12,
	viewJustify : vjRightH,
	wrapAround : true,
	ClickDone : function() {
		this._parent.SetHours(this.value);
	}
}), protoTimeTyper, 'hours');
DeclareChild(View({
	_proto: protoText,
	viewJustify : vjCenterH,
	text : ':'
}), protoTimeTyper, 'colon');
DeclareChild(View({
	_proto: protoNumberTyper,
	minValue : 0,
	maxValue : 59,
	viewJustify : vjCenterH,
	wrapAround : true,
	showLeadingZeros : true,
	ClickDone : function() {
			this._parent.SetMinutes(this.value);
		}
}), protoTimeTyper, 'minutes');
DeclareChild(View({
	_proto : protoListTyper,
	viewJustify : vjCenterH,
	chromeStyle : 'monoFont mediumFont numberTyper',
	pickItems: ['AM','PM'],
	ClickDone : function() {
		this._parent.SetAmPm(this.value);
	}
}), protoTimeTyper, 'ampm');


protoDateTyper = View({
	_proto : protoView,
	viewClass : 'dateTyper',
	date : null,
	MonthFormatter : function(monthNum) {
		return this.month.pickItems[monthNum];
	},
	MonthLength : function(monthNum, year) {
		var monthLength = [31,28,31,30,31,30,31,31,30,31,30,31][monthNum];
		if (monthNum == 1) {
			if ((year % 4) == 0) {
				// perhaps leap year - not on centuries unless divisible by 400
				if (year % 100 != 0 || year % 400 == 0) {
					monthLength = 29;
				}
			}
		}
		return monthLength;
	},
	RefreshDate : function() {
		SetValue(this.dayOfMonth, 'value', this.date.getDate());
		SetValue(this.month, 'value', this.MonthFormatter(this.date.getMonth()));
		SetValue(this.year, 'value', this.date.getFullYear());
		SetValue(this.dayOfMonth, 'maxValue', this.MonthLength(this.date.getMonth(), this.date.getFullYear()));
	},
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		if (!this.date) {
			this.date = new Date();
		}
		//this.RefreshDate();
		this.LayoutChildrenMatrix({
			tabAcross: 5,
			tabDown: 1,
			tabWidths: this.chromeChildWidths,
			tabHeights: this.chromeHeight - 2,
			tabChildren: [this.dayOfMonth, null, this.month,null, this.year]
		},1,1);
	},
	chromeHeight : 18,
	chromeChildWidths : [20,/* Days */
	5,	/* : */
	30,	 /* month */
	5,	/* */
	40	/* year */
	],
	chromeWidth : 20 + 5 + 30 + 5 + 40 + 2,
	SetMonth : function(value) {
		var m = this.month.pickItems.findFirst(value);
		if (m !== null) {
			var day = this.date.getDate();
			if (day > this.MonthLength(m, this.date.getFullYear())) {
				// limit the day
				alert('limit day to ' + this.MonthLength(m, this.date.getFullYear()));
				this.date.setDate(this.MonthLength(m, this.date.getFullYear()));
			}
			this.date.setMonth(m);
			this.RefreshDate();
			this.NewDate(this.date);
		}
	},
	SetDayOfMonth : function(value) {
		this.date.setDate(value);
		this.NewDate(this.date);
	},
	SetYear : function(value) {
		var day = this.date.getDate();
		if (day > this.MonthLength(this.date.getMonth(), value)) {
			// limit the day
			this.date.setDate(this.MonthLength(this.date.getMonth(), value));
		}
		this.date.setFullYear(value);
		this.RefreshDate();
		this.NewDate(this.date);
	},
	ViewChangedScript : function(symbol,value) {
		if (symbol == 'date') {
			// format the value
			this.date = value;
			this.RefreshDate();
		}
	},
	NewDate : function(date) {
	}
});
DeclareChild(View({
	_proto: protoNumberTyper,
	minValue : 1,
	maxValue : 31,
	viewJustify : vjRightH,
	wrapAround : true,
	ClickDone : function() {
		this._parent.SetDayOfMonth(this.value);
	}
}), protoDateTyper, 'dayOfMonth');
DeclareChild(View({
	_proto : protoListTyper,
	viewJustify : vjCenterH,
	chromeStyle : 'monoFont mediumFont numberTyper',
	pickItems: ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'],
	ClickDone : function() {
		this._parent.SetMonth(this.value);
	}
}), protoDateTyper, 'month');
DeclareChild(View({
	_proto: protoNumberTyper,
	minValue : 1904,
	maxValue : 2034,
	viewJustify : vjCenterH,
	wrapAround : true,
	showLeadingZeros : true,
	ClickDone : function() {
		this._parent.SetYear(this.value);
	},
	KeyEvent : function(event) {
		event.stopPropagation();
		if (48 <= event.charCode && event.charCode <= 57) {
			// years are tricky in the range of 1904..2034
			this.PrepareForClick();
			var digit = event.charCode - 48;
			var value = (this.value % 10) * 10 + digit;
			if (value <= 34) {
				value = 2000 + value;
			} else {
				value = 1900 + value;
			}
			if (value > this.maxValue) {
				value = digit; // reset to single digit?
			}
			if (value > this.maxValue) {
				value = this.minValue;
			}
			SetValue(this, 'value', value);
			this.ClickDone();
			return true;
		}
		return protoNumberTyper.KeyEvent.call(this, event);
	}

}), protoDateTyper, 'year');


if (0) {
protoLongLatTextTyper = View({
	_proto : protoView,
	latitude : 0,
	longitude : 0,
	ViewSetupChildrenScript : function() {
		protoView.ViewSetupChildrenScript.call(this);
		this.hours.viewJustify = vjParentLeftH + vjRightH;
		this.hours.viewBounds = RelBounds(0,0, this.chromeChildWidths[0], this.chromeHeight);
		this.colon.viewJustify = vjSiblingRightH + vjCenterH;
		this.colon.viewBounds = RelBounds(0,0, this.chromeChildWidths[1], this.chromeHeight);
		this.minutes.viewJustify = vjSiblingRightH + vjCenterH;
		this.minutes.viewBounds = RelBounds(0,0, this.chromeChildWidths[2], this.chromeHeight);
		this.ampm.viewJustify = vjSiblingRightH + vjCenterH;
		this.ampm.viewBounds = RelBounds(this.chromeChildWidths[3],0, this.chromeChildWidths[4], this.chromeHeight);
	},
	ViewSetupDoneScript : function() {
		this.ChangeLatitude();
		this.ChangeLongitude();
	},
	chromeHeight : 20,
	chromeChildWidths : [20,/* Degrees */
	5,	/* deg */
	25,	 /* Minutes */
	5,	/* */
	30, /* N/S */
	10, /* */
	20,/* Degrees */
	5,	/* deg */
	25,	 /* Minutes */
	5,	/* */
	30 /* E/W */
	],
	chromeWidth : 20 + 5 + 25 + 5 + 30,
	NormalizeRelDegrees : function(value) {
		var deg = value % 360;
		if (value < 0) {
			deg = 360 - (-value) % 360;
		}
		if (deg <= 180) {
			return value;
		} else {
			return -(360 - value);
		}
	},
	ChangeLatitude : function() {
		if (this.latitudeNS.value ==  'N') {
			this.latitude = Math.abs(this.latitudeDegrees + this.latitudeMinutes / 60);
		} else {
			this.latitude = -Math.abs(this.latitudeDegrees + this.latitudeMinutes / 60);
		}
		this.latitude = this.NormalizeRelDegrees(this.latitude);
		if (this.latitude >= 0) {
			SetValue(this.latitudeDegrees,'value', Math.floor(this.latitude));
			SetValue(this.latitudeMinutes,'value', Math.floor((this.latitude - Math.floor(this.latitude)) * 60));
			SetValue(this.latitudeNS,'value','N');
		} else {
			SetValue(this.latitudeDegrees,'value', Math.floor(-this.latitude));
			SetValue(this.latitudeMinutes,'value', Math.floor(((-this.latitude) - Math.floor(-this.latitude)) * 60));
			SetValue(this.latitudeNS,'value','S');
		}
	},
	ChangeLongitude : function() {
		if (this.longitudeNS.value ==  'N') {
			this.longitude = Math.abs(this.longitudeDegrees + this.longitudeMinutes / 60);
		} else {
			this.longitude = -Math.abs(this.longitudeDegrees + this.longitudeMinutes / 60);
		}
		this.longitude = this.NormalizeRelDegrees(this.longitude);
		if (this.longitude >= 0) {
			SetValue(this.longitudeDegrees,'value', Math.floor(this.longitude));
			SetValue(this.longitudeMinutes,'value', Math.floor((this.longitude - Math.floor(this.longitude)) * 60));
			SetValue(this.longitudeNS,'value','N');
		} else {
			SetValue(this.longitudeDegrees,'value', Math.floor(-this.longitude));
			SetValue(this.longitudeMinutes,'value', Math.floor(((-this.longitude) - Math.floor(-this.longitude)) * 60));
			SetValue(this.longitudeEW,'value','S');
		}
	}
});
protoRelativeDegreeTyper = View({
	_proto: protoNumberTyper,
	minValue : -179,
	maxValue : 180,
	wrapAround : true,
	ViewChangedScript : function(symbol,value) {
		if (symbol == 'value') {
			// format the value
			this.viewElement.innerText = Math.abs(protoLongLatTextTyper.NormalizeRelDegrees(this.value)) + "";
		}
	}
});
DeclareChildView(View({_proto: protoRelativeDegreeTyper,
	ClickDone: function() {
		this._parent.ChangeLongitude();
	}
}), protoLongLatTextTyper, 'longituteDegrees');
DeclareChild(View({
	_proto: protoNumberTyper,
	minValue : 0,
	maxValue : 59,
	wrapAround : true,
	showLeadingZeros : true,
	ClickDone : function() {
			this._parent.ChangeLongitude();
		}
}), protoTimeTyper, 'longitudeMinutes');
DeclareChild(View({
	_proto : protoListTyper,
	pickItems: ['E','W'],
	ClickDone : function() {
		this._parent.ChangeLongitude();
	}
}), protoTimeTyper, 'longitudeEW');
DeclareChildView(View({_proto:protoRelativeDegreeTyper,
	ClickDone: function() {
		this._parent.ChangeLatitude();
	}
}), protoLongLatTextTyper, 'latitudeDegrees');
DeclareChild(View({
	_proto: protoNumberTyper,
	minValue : 0,
	maxValue : 59,
	wrapAround : true,
	showLeadingZeros : true,
	ClickDone : function() {
			this._parent.ChangeLatitude();
		}
}), protoTimeTyper, 'latitudeMinutes');
DeclareChild(View({
	_proto : protoListTyper,
	pickItems: ['N','S'],
	ClickDone : function() {
		this._parent.ChangeLatitude();
	}
}), protoTimeTyper, 'latitudeNS');
}